"use strict";
this.name	= "separatedlasers";
this.author	= "Norby";
this.copyright	= "2016 Norby";
this.description= "Lasers will not heat up faster if there is at least 30m space between multiple mounts";
this.licence	= "CC BY-NC-SA 4.0";


//worldscript events
this.startUpComplete = this.playerBoughtNewShip = function() {
    this._Replace_Lasers(player.ship, this); //check and replace lasers to separated if possible
}

this.shipSpawned = function(ship) {
    this._Replace_Lasers(ship, this); //check and replace NPC lasers to separated if possible
}

this.playerBoughtEquipment = function( equipment ) {
    if( equipment.indexOf("EQ_WEAPON_") > -1 ) { //core lasers, laser cannons and other lasers in the future
        //check and replace new laser to separated
        //and replace back to basic laser if bought to not separated side
        this._Replace_Lasers(player.ship, this);
    }
}

//condition script
this.allowAwardEquipment = function(equipment, ship, context)
{
        // OXP hook to allow stations to forbid specific equipment
        if (context == "purchase" && player.ship.dockedStation && player.ship.dockedStation.scriptInfo["oolite-barred-equipment"])
        {
                if (player.ship.dockedStation.scriptInfo["oolite-barred-equipment"].indexOf(equipment) != -1)
                {
                        return false;
                }
        }

        // OXP hook to allow ships to forbid specific "available to all" equipment
        if (ship.scriptInfo && ship.scriptInfo["oolite-barred-equipment"] && ship.scriptInfo["oolite-barred-equipment"].indexOf(equipment) != -1)
        {
                return false;
        }

//      player.consoleMessage( eqKey+" "+ship+" "+context );//debug


        //Eco Artillery need as large ship as an Anaconda
        if( ship.mass < 400000 && equipment.indexOf("EQ_WEAPON_CANNON_EA") > -1 ) return false;

        //Eco Reactor, Vulcan Cannon and Plasma Web need as large ship as a Python
        if( ship.mass < 250000 &&
                (  equipment.indexOf("EQ_WEAPON_ECO_REACTOR") > -1
                || equipment.indexOf("EQ_WEAPON_CANNON_7_") > -1
                || equipment.indexOf("EQ_WEAPON_CANNON_9_") > -1 ) ) return false;

        //Eco Laser, Mini Cannon and Mini Plasma Ray fit into escort ships from 30t
        if( ship.mass <= 30000 &&
                (  equipment.indexOf("EQ_WEAPON_ECO_LASER") > -1
                || equipment.indexOf("EQ_WEAPON_CANNON_2_") > -1
                || equipment.indexOf("EQ_WEAPON_CANNON_8_") > -1 ) ) return false;

        //Other cannons for large ships only like Cobra Mark III
        if( equipment.indexOf("EQ_WEAPON_CANNON") > -1 && ship.mass < 130000 ) return false;

        return true;
}


//internal methods
this._Replace_Lasers = function(ship, w) {  //check and replace lasers to separated if possible
    if( !w ) var w = worldScripts.separatedlasers;
    var a = this._Separated_Lasers(ship, w);
    if( a ) {
        log(this.name, ship.name+" Separated laser distances: fw:"+a[1]+" aft:"+a[2]+" port:"+a[3]+" sb:"+a[4]);
        //todo: replace Pulse, Beam and Military lasers to separated versions where distance is true
        //Dual, Triple, Quad or Broad Separated Pulse, Beam or Military Lasers and some Laser Cannons

        //use the laser match to the number of mounts but max.no.5


        //replace back to basic laser if bought to not separated side

    }
}

this._Separated_Lasers = function(ship, w) {  //return false if no separated mount, else return array of sides
    if( !w ) var w = worldScripts.separatedlasers;
    var a = [];
    a[1] = w._is_Separated_Mount(ship, 1, w); //forward
    a[2] = w._is_Separated_Mount(ship, 2, w); //aft
    a[3] = w._is_Separated_Mount(ship, 3, w); //port
    a[4] = w._is_Separated_Mount(ship, 4, w); //starboard
//  log(this.name, ship.name+" Laser distances: fw:"+a[1]+" aft:"+a[2]+" port:"+a[3]+" sb:"+a[4]);
    if( !a[1] && !a[2] && !a[3] && !a[4] ) return false;
    else return a; //array of min. distances at sides where lasers are separated, false at other sides
}

this._is_Separated_Mount = function(ship, side, w) { //return false if this side is not separated
    if( !w ) var w = worldScripts.separatedlasers;
    var d = w._mount_Distance(ship, side, w);
    if( d < 30 ) return false;
    else return d; //return min. distance if more than the minimally allowed for separated lasers
}

this._mount_Distance = function(ship, side, w) { //return 0 if only one mount, min. dist. if more.
    if( !w ) var w = worldScripts.separatedlasers;
    side = Math.floor(side);
    var array;
    if( !side || side <= 1 || side > 4 ) array = ship.weaponPositionForward;
    else if( side == 2 ) array = ship.weaponPositionAft;
    else if( side == 3 ) array = ship.weaponPositionPort;
    else array = ship.weaponPositionStarboard;
    var min = w._min_Distance(array);
    return min;
}

this._min_Distance = function(array) { //calculate min. dist. in the given array of Vector3Ds
    var min = 0;
    if( array && array.length > 1 ) for(var i = 0; i < array.length; i++) {
      for(var j = i + 1; j < array.length; j++) { //check against all remaining items
        var d = 0;
        if( array[i].distanceTo && array[j].distanceTo ) d = array[i].distanceTo(array[j]);
        if( d > 0 && min == 0 ) min = d;
        else if( min > 0 && d < min ) min = d;
//      log(this.name, "d:"+d+"min:"+min+" a["+i+"]: "+array[i]+" a["+j+"]:"+array[j]);
      }
    }
//  log(this.name, " array: "+array+" min.d:"+min);
    return min;
}
